package edu.illinois.reassert.assertfixer;

import java.lang.reflect.Method;

import spoon.reflect.code.CtInvocation;
import edu.illinois.reassert.AssertFactory;
import edu.illinois.reassert.AssertFixer;
import edu.illinois.reassert.CodeFixResult;
import edu.illinois.reassert.Factory;
import edu.illinois.reassert.FixResult;
import edu.illinois.reassert.UnfixableException;

/**
 * Intermediate strategy that simply changes assertNull(x) to assertEquals(null, x)
 * so other strategies can fix it appropriately.  
 * 
 * @author <a href="http://www.cs.uiuc.edu/homes/bdaniel3/">Brett Daniel</a>
 */
public class AssertNullToAssertEqualsFixer extends AssertFixer {

	public AssertNullToAssertEqualsFixer(Factory factory) {
		super(factory);
	}

	@Override
	public FixResult fix(
			Method testMethod, 
			CtInvocation<?> assertion,
			Throwable failureException) throws UnfixableException {
		AssertFactory assertFactory = getAssertFactory(testMethod);
		if (assertFactory.isAssertNull(assertion)) {
			// create assertEquals(null, actual) and fix on next pass
			CtInvocation<?> newAssert = assertFactory.createAssertEquals(
					getFactory().Code().createLiteral(null),
					assertion.getArguments().get(0));
			replaceStatement(assertion, newAssert);
			return new CodeFixResult(newAssert);			
		}
		return null;
	}

}
