package edu.illinois.reassert.test;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import org.junit.Before;
import org.junit.Test;

import spoon.reflect.code.CtExpression;
import spoon.reflect.cu.SourceCodeFragment;
import spoon.reflect.declaration.CtClass;
import edu.illinois.reassert.Factory;
import edu.illinois.reassert.SimpleSpoonLoader;


public class SimpleSpoonLoaderTest {
	private static final String TEST_DIR = "test";
	private SimpleSpoonLoader loader;
	private Factory factory;
	
	@Before
	public void init() throws IOException {
		factory = new Factory();
		factory.addSourcePath(TEST_DIR);
		loader = new SimpleSpoonLoader(factory);
	}
	
	@SuppressWarnings("unchecked")
	@Test
	public void testLoadModifyReload() throws IllegalArgumentException, SecurityException, IllegalAccessException, ClassNotFoundException, IOException {
		CtClass<?> reflection = (CtClass<?>) factory.Class().get(ClassToModify.class);
		
		CtExpression<?> expr = reflection.getFields().get(0).getDefaultExpression();
		int exprStart = expr.getPosition().getSourceStart();
		int exprEnd = expr.getPosition().getSourceEnd();
		
		SourceCodeFragment frag = new SourceCodeFragment();
		frag.position = exprStart;
		frag.replacementLength = exprEnd - exprStart + 1;
		frag.code = "\"MODIFIED\"";
		expr.getPosition().getCompilationUnit().addSourceCodeFragment(frag);		

		Class modified = loader.load(ClassToModify.class.getName());
		assertEquals("MODIFIED", modified.getFields()[0].get(null));
		
		SourceCodeFragment frag2 = new SourceCodeFragment();
		frag2.position = exprStart;
		frag2.replacementLength = exprEnd - exprStart + 1;
		frag2.code = "\"M______D\"";
		expr.getPosition().getCompilationUnit().getSourceCodeFraments().clear();
		expr.getPosition().getCompilationUnit().addSourceCodeFragment(frag2);		
		
		Class modified2 = loader.load(ClassToModify.class.getName());
		assertEquals("M______D", modified2.getFields()[0].get(null));
	}

	public static class ClassToModify {
		public static String field = "ORIGINAL";
	}
}
